
<template>
    <div class="s-explorer-pane" style="width:50%">

        <div class="row">
            <div class="col-md-6">
                <h3>Reports</h3>
            </div>
            <div class="col-md-5">
                <b-form-fieldset horizontal label="Filter" :label-cols="3">
                    <b-form-input v-model="filter" placeholder="Type to Search" />
                </b-form-fieldset>
            </div>
            <div class="col-md-1">
                <b-btn v-b-modal.modal1>...</b-btn>
            </div>
        </div>

        <b-table class="table" :style="{minWidth: width}" small v-model="items" :items="rows" :fields="columns" :filter="filter" @row-clicked="rowClicked" @mousedown.native.shift.prevent>
        </b-table>

        <b-modal id="modal1" title="Select Columns" size="lg" @ok="okModal">
            <b-container fluid>
                <b-row>
                    <b-col v-for="section in ['case', 'sample', 'readout']">
                        <div v-for="col in initial" v-if="col.section == section">
                            <b-form-checkbox v-model="col.selected">{{col.label}}</b-form-checkbox>
                        </div>
                    </b-col
                </b-row>
            </b-container>
        </b-modal>
    </div>
</template>

<style>

table.table {
    border-bottom: 1px solid #eceeef;
}

table.table tr:hover {
    background: #eee;
}

table.table td {
    vertical-align: middle;
}

table.report td {
    padding: 0.1em 0.7em 0.1em 0;
    border: none;
}

</style>

<script>


function flatten(attrs){

    function key(i, j){

        var section = attrs[i];

        if (!section){
            section = {};
            attrs[i] = section;
        }

        var name = section[j];

        if (!name){
            name = i + '_' + j;
            section[j] = name;
        }

        return name;
    }

    return function(source){

        var target = {};

        Object.keys(source).forEach(function(i){

            var value = source[i];

            if (typeof value != 'object'){
                target[i] = value;
            }
            else if (!Array.isArray(value)){
                Object.keys(value).forEach(function(j){
                    target[key(i, j)] = value[j];
                });
            }
        });

        return target;
    };
}


function completed(item){
    return item.status == 'completed';
}


function loadSelected(){

    try {

        var selected = {}, columns = [], stored = localStorage.getItem('explorer-columns');

        if (stored) {
            columns = JSON.parse(stored);
        }
        else {
            return null;
        }

        columns.forEach(function(col){
            selected[col.key] = col.selected;
        });

        return selected;
    }
    catch(err){
        return null;
    }
}


function toColumns(attrs){

    var defaultSelected = {
        case_id: true,
        case_condition: true,
        sample_id: true,
        sample_type: true,
        readout_isolate: true
    };

    var section, name, columns = [], selected = loadSelected() || defaultSelected;

    for(section in attrs){
        for(name in attrs[section]){
            columns.push({
                key: attrs[section][name],
                section: section,
                label: name == 'id' ? section : name,
                sortable: true,
                selected: selected[attrs[section][name]] || false
            });
        }
    }

    return columns;
}


function markSelectedRows(rows, id){

    var list = id.split(',');

    rows.forEach(function(row){
        row._rowVariant = (list.indexOf(String(row.id)) > -1) ? 'active' : '';
    });
}


function navigate(router, id){
    router.push((String(id).indexOf(',') > -1 ? '/explorer/meta/' : '/explorer/') + id);
}


function selectedRows(item, rows, selection, ctrl, shift){

    var id = String(item.id),
        sel = selection.split(',');

    function notEqual(s){
        return s != id;
    }

    if (!ctrl && !shift){
        return id;
    }

    if (!shift){
        return sel.every(notEqual) ? sel.concat(id) : sel.filter(notEqual);
    }

    sel.push(id);

    var min = rows.length,
        max = -1;

    rows.forEach(function(row, index){
        if (sel.indexOf(String(row.id)) > -1){
            min = Math.min(min, index);
            max = Math.max(max, index);
        }
    });

    sel = [];

    rows.forEach(function(row, index){
        if (index >= min && index <= max){
            sel.push(String(row.id));
        }
    });

    return sel;
}


function selected(col){
    return col.selected;
}


function created(){

    var row = this.rows[0];

    if (row && this.id == '-') {
        navigate(this.$router, row.id);
    }
}


var attributes = {
    case: {
        'id': 'case_id',
        'origin': 'case_origin',
        'condition': 'case_condition',
        'contact information': 'case_contact information',
        'location': 'case_location'
    },

    sample: {
        'id': 'sample_id',
        'type': 'sample_type',
        'processing': 'sample_processing',
        'time': 'sample_time'
    },

    readout: {
        'isolate': 'readout_isolate',
        'method': 'readout_method',
        'sequencing instrument': 'readout_sequencing instrument',
        'PE fragment size': 'readout_PE fragment size'
    }
};


function data(){

    var columns = toColumns(attributes);

    return {
        items: [],
        columns: columns.filter(selected),
        initial: columns,
        selected: '',
        filter: ''
    };
}


var computed = {

    rows: function(){
        var rows = this.$root.analyses.filter(completed).map(flatten(attributes));
        markSelectedRows(rows, this.id);
        return rows;
    },

    width: function(){
        return this.columns.length * 8 + 'em';
    }
};


var watch = {

    id: function(id){

        var row = this.rows[0];

        if (row && id == '-') {
            navigate(this.$router, row.id);
            return;
        }

        markSelectedRows(this.rows, id);
    }
};


var methods = {
    rowClicked: function(item, index, event){
        navigate(this.$router, selectedRows(item, this.items, this.id, event.ctrlKey, event.shiftKey));
    },

    okModal: function(){
        this.columns = this.initial.filter(selected);
        localStorage.setItem('explorer-columns', JSON.stringify(this.initial));
    }
};


module.exports = {
    props: ['id'],
    data: data,
    computed: computed,
    watch: watch,
    methods: methods,
    created: created
};


</script>